create or replace PACKAGE ETLPROCEDURES AS 

  --Procediments per carregar les dades a les taules de dimensions
  procedure dwLoadFranjaEdatETL;
  procedure dwLoadPaisETL;
  procedure dwLoadSexeETL;
  procedure dwLoadPlataformaETL;
  procedure dwLoadTempsETL(m NUMBER, y NUMBER);  
  
  --Procediments per carregar les dades a les taules de fets
  procedure dwLoadVidaETL(m NUMBER, y NUMBER);
  procedure dwLoadPartidaETL(m NUMBER, y NUMBER);
  procedure dwLoadAjudaETL(m NUMBER, y NUMBER);
  

END ETLPROCEDURES;

create or replace PACKAGE BODY ETLPROCEDURES AS

  procedure dwLoadFranjaEdatETL AS
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadFranjaEdatETL';
    
    BEGIN
      delete from dw_franjaedat;
      insert into dw_franjaedat(id, edatInferior, edatSuperior) values (1, 18, 21);
      insert into dw_franjaedat(id, edatInferior, edatSuperior) values (2, 22, 30);
      insert into dw_franjaedat(id, edatInferior, edatSuperior) values (3, 31, 45);
      insert into dw_franjaedat(id, edatInferior, edatSuperior) values (4, 46, 60);
      insert into dw_franjaedat(id, edatInferior, edatSuperior) values (5, 61, 200);
      
      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);    
          
  END dwLoadFranjaEdatETL;
  
  --------------------------------------------------------------------------------------------------------  

  procedure dwLoadPaisETL AS
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadPaisETL';
    CURSOR paisosCur IS select id, nom from PFC.pais;
    idPais NUMBER;
    nomPais VARCHAR2(100);
    
    BEGIN
    
      open paisosCur;
      LOOP
        FETCH paisosCur INTO idPais, nomPais;
        EXIT WHEN paisosCur%NOTFOUND;
        insert into DW_Pais(id, nom) values (idPais, nomPais);        
      END LOOP;

      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);    
  END dwLoadPaisETL;

  --------------------------------------------------------------------------------------------------------  

  procedure dwLoadSexeETL AS
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadSexeETL';
    
    BEGIN
      delete from DW_Sexe;
      insert into DW_Sexe(id, descripcio) values (0, 'Home');
      insert into DW_Sexe(id, descripcio) values (1, 'Dona');
      
      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);    
  END dwLoadSexeETL;
  
  --------------------------------------------------------------------------------------------------------  

  procedure dwLoadPlataformaETL AS
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadPlataformaETL';
    CURSOR plataformesCur IS select id, nom from PFC.plataforma;
    idPlataforma NUMBER;
    nomPlataforma VARCHAR2(100);
    
    BEGIN
    
      open plataformesCur;
      LOOP
        FETCH plataformesCur INTO idPlataforma, nomPlataforma;
        EXIT WHEN plataformesCur%NOTFOUND;
        insert into DW_Plataforma(id, nom) values (idPlataforma, nomPlataforma);        
      END LOOP;

      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);   
  END dwLoadPlataformaETL;
  
  --------------------------------------------------------------------------------------------------------    
  
  procedure dwLoadTempsETL(m NUMBER, y NUMBER) AS  
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadTempsETL';
   
    BEGIN
      parametres('mes') := m;
      parametres('year') := y;
    
      insert into DW_Temps(mes, year) values (m, y);

      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);    
  
  END dwLoadTempsETL;
  
  --------------------------------------------------------------------------------------------------------  

  procedure dwLoadVidaETL(m NUMBER, y NUMBER) AS
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadVidaETL';
    CURSOR franjaEdatCur IS select * from DW_FranjaEdat;
    CURSOR paisCur IS select * from DW_Pais;
    CURSOR sexeCur IS select * from DW_Sexe;
    franjaEdatRec franjaEdatCur%rowtype;
    paisRec paisCur%rowtype;
    sexeRec sexeCur%rowtype;
    videsComprades NUMBER;
    sumaImportVides NUMBER;
    videsIntercanviades NUMBER;
    idTemps_ NUMBER;
    
    BEGIN
      parametres('mes') := m;
      parametres('year') := y;
      
      select id into idTemps_ from DW_Temps where mes=m and year=y; 
      delete from DW_Vida where idTemps = idTemps_;
      
      open franjaEdatCur;
      LOOP  --Cursor de Franja d'edat
        FETCH franjaEdatCur INTO franjaEdatRec;
        EXIT WHEN franjaEdatCur%NOTFOUND;        
        
        open paisCur;
        LOOP  --Cursor de Pais
          FETCH paisCur INTO paisRec;
          EXIT WHEN paisCur%NOTFOUND;          
          
          open sexeCur;
          LOOP  --Cursor de Sexe              
            FETCH sexeCur INTO sexeRec;
            EXIT WHEN sexeCur%NOTFOUND;
            
            select count(*), sum(v.import) into videsComprades, sumaImportVides from PFC.Vida v, PFC.Usuari u where 
            (v.usuaripropietariid = u.id) and
            (extract(year from v.dataCarrega) = y) and
            (extract(month from v.dataCarrega) = m) and
            (floor(months_between(sysdate,u.datanaixement)/12) >= franjaEdatRec.edatInferior) and
            (floor(months_between(sysdate,u.datanaixement)/12) <= franjaEdatRec.edatSuperior) and
            (u.sexe = sexeRec.id) and (u.paisId = paisRec.id) and
            (v.tipusVidaId = 3);
            
            select count(*) into videsIntercanviades from PFC.Vida v, PFC.Usuari u where 
            (v.usuaripropietariid = u.id) and
            (extract(year from v.dataCarrega) = y) and
            (extract(month from v.dataCarrega) = m) and
            (floor(months_between(sysdate,u.datanaixement)/12) >= franjaEdatRec.edatInferior) and
            (floor(months_between(sysdate,u.datanaixement)/12) <= franjaEdatRec.edatSuperior) and
            (u.sexe = sexeRec.id) and (u.paisId = paisRec.id) and
            (v.tipusVidaId = 2);              
            
            if(videsComprades is null) then videsComprades:=0; end if;
            if(sumaImportVides is null) then sumaImportVides:=0; end if;
            if(videsIntercanviades is null) then videsIntercanviades:=0; end if;
            
            insert into DW_Vida(IdPais, IdFranjaEdat, IdSexe, IdTemps, comprades, sumaImport, intercanviades) 
            values (paisRec.id, franjaEdatRec.id, sexeRec.id, idTemps_, videsComprades, sumaImportVides, videsIntercanviades);              
          END LOOP;
          close sexeCur;
        END LOOP;
        close paisCur;
      END LOOP;
      close franjaEdatCur;

      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);  
  END dwLoadVidaETL;
  
  --------------------------------------------------------------------------------------------------------
  
  procedure dwLoadPartidaETL(m NUMBER, y NUMBER) AS
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadPartidaETL';
    CURSOR franjaEdatCur IS select * from DW_FranjaEdat;
    CURSOR paisCur IS select * from DW_Pais;
    CURSOR sexeCur IS select * from DW_Sexe;
    CURSOR plataformaCur IS select * from DW_Plataforma;    
    franjaEdatRec franjaEdatCur%rowtype;
    paisRec paisCur%rowtype;
    sexeRec sexeCur%rowtype;
    plataformaRec plataformaCur%rowtype;
    partidesJugades NUMBER;
    horesDeJoc NUMBER;
    idTemps_ NUMBER;
    
    BEGIN
      parametres('mes') := m;
      parametres('year') := y;
      
      select id into idTemps_ from DW_Temps where mes=m and year=y; 
      delete from DW_Partida where idTemps = idTemps_;
      
      open franjaEdatCur;
      LOOP  --Cursor de Franja d'edat
        FETCH franjaEdatCur INTO franjaEdatRec;
        EXIT WHEN franjaEdatCur%NOTFOUND;        
        
        open paisCur;
        LOOP  --Cursor de Pais
          FETCH paisCur INTO paisRec;
          EXIT WHEN paisCur%NOTFOUND;          
          
          open sexeCur;
          LOOP  --Cursor de Sexe              
            FETCH sexeCur INTO sexeRec;
            EXIT WHEN sexeCur%NOTFOUND;
            
            open plataformaCur;
            LOOP  --Cursor de Plataforma           
            FETCH plataformaCur INTO plataformaRec;
            EXIT WHEN plataformaCur%NOTFOUND;
            
              select count(*), sum((p.dataFinal-p.dataInici)*24) into partidesJugades, horesDeJoc from PFC.Partida p, PFC.Usuari u where
                (p.usuariId = u.id) and
                (p.plataformaId = plataformaRec.id) and
                (p.dataFinal is not null) and
                (extract(year from p.dataInici) = y) and
                (extract(month from p.dataInici) = m) and
                (floor(months_between(sysdate,u.datanaixement)/12)>=franjaEdatRec.edatInferior) and
                (floor(months_between(sysdate,u.datanaixement)/12)<=franjaEdatRec.edatSuperior) and
                (u.sexe = sexeRec.id) and (u.paisId = paisRec.id);  
              
              if(partidesJugades is null) then partidesJugades:=0; end if;
              if(horesDeJoc is null) then horesDeJoc:=0; end if;
              
              insert into DW_Partida(IdFranjaEdat, IdPais, IdTemps, IdSexe, IdPlataforma, jugades, horesDeJoc) 
              values (franjaEdatRec.id, paisRec.id, idTemps_, sexeRec.id, plataformaRec.id, partidesJugades, horesDeJoc);              
            END LOOP;
            close plataformaCur;
          END LOOP;
          close sexeCur;
        END LOOP;
        close paisCur;
      END LOOP;
      close franjaEdatCur;

      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage); 
  END dwLoadPartidaETL;

  --------------------------------------------------------------------------------------------------------
  
  procedure dwLoadAjudaETL(m NUMBER, y NUMBER) AS
    parametres PFC.logpackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dwLoadAjudaETL';
    CURSOR franjaEdatCur IS select * from DW_FranjaEdat;
    CURSOR paisCur IS select * from DW_Pais;
    CURSOR sexeCur IS select * from DW_Sexe;
    franjaEdatRec franjaEdatCur%rowtype;
    paisRec paisCur%rowtype;
    sexeRec sexeCur%rowtype;
    ajudesComprades NUMBER;
    sumaImportAjudes NUMBER;
    idTemps_ NUMBER;
    iteracio NUMBER;
    
    BEGIN
      parametres('mes') := m;
      parametres('year') := y;
      iteracio := 0;
      
      select id into idTemps_ from DW_Temps where mes=m and year=y; 
      delete from DW_Vida where idTemps = idTemps_;
      
      open franjaEdatCur;
      LOOP  --Cursor de Franja d'edat
        FETCH franjaEdatCur INTO franjaEdatRec;
        EXIT WHEN franjaEdatCur%NOTFOUND;        
        
        open paisCur;
        LOOP  --Cursor de Pais
          FETCH paisCur INTO paisRec;
          EXIT WHEN paisCur%NOTFOUND;          
          
          open sexeCur;
          LOOP  --Cursor de Sexe              
            FETCH sexeCur INTO sexeRec;
            EXIT WHEN sexeCur%NOTFOUND;
            iteracio:=iteracio + 1;
            
            select count(*), sum(au.import) into ajudesComprades, sumaImportAjudes from PFC.AjudaUsuari au, PFC.Usuari u where 
            (au.usuariId = u.id) and
            (extract(year from au.dataCompra) = y) and
            (extract(month from au.dataCompra) = m) and
            (floor(months_between(sysdate,u.datanaixement)/12) >= franjaEdatRec.edatInferior) and
            (floor(months_between(sysdate,u.datanaixement)/12) <= franjaEdatRec.edatSuperior) and
            (u.sexe = sexeRec.id) and (u.paisId = paisRec.id);
                          
            if(ajudesComprades is null) then ajudesComprades:=0; end if;
            if(sumaImportAjudes is null) then sumaImportAjudes:=0; end if;
            
            insert into DW_Ajuda(IdFranjaEdat, IdPais, IdTemps, IdSexe, comprades, sumaImport) 
            values (franjaEdatRec.id, paisRec.id, idTemps_, sexeRec.id, ajudesComprades, sumaImportAjudes);
            commit;
          END LOOP;
          close sexeCur;
        END LOOP;
        close paisCur;
      END LOOP;
      close franjaEdatCur;

      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      PFC.logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, 'Iteracio: ' || iteracio);   
      
      EXCEPTION
        WHEN others THEN      
          ROLLBACK;
          errorMessage := SQLERRM;
          PFC.LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);  
  END dwLoadAjudaETL;

END ETLPROCEDURES;