create or replace PACKAGE LOGPACKAGE AS 
    TYPE parametresList IS TABLE OF VARCHAR2(500) INDEX BY VARCHAR2(50);
    TYPE logRec IS RECORD (
      id NUMBER,
      data DATE,
      procediment VARCHAR2(50),
      parametres VARCHAR2(1000),
      resultat NUMBER,
      missatge VARCHAR2(4000)
    );
    
    TYPE llistaLogsTable is table of logRec;
    
    PROCEDURE altaLogEntry(data IN DATE, procediment IN VARCHAR2, parametres IN parametresList, resultat IN NUMBER, missatge IN VARCHAR2);
    function consultaLogs(dataInici date, dataFinal date) return llistaLogsTable;
    procedure esborraLogs(dataInici date, dataFinal date);

END LOGPACKAGE;

create or replace PACKAGE BODY LOGPACKAGE AS

  /*
  * altaLogEntry dna d'alta un registre a la taula de Logs amb el resultat d'una crida a un procediment. 
  */
  PROCEDURE altaLogEntry(data IN DATE, procediment IN VARCHAR2, parametres IN parametresList, resultat IN NUMBER, missatge IN VARCHAR2) AS
    nomParametre VARCHAR2(50);
    parametresString VARCHAR(5000); --Llista de parametres passats a string
    BEGIN
      nomParametre := parametres.LAST;      
      parametresString := '';
      while nomParametre is not null loop
        parametresString := parametresString || nomParametre || ': ' || parametres(nomParametre) || '; ';
        nomParametre := parametres.PRIOR(nomParametre);
      end loop;
      
      INSERT INTO Log(data, procediment, parametres, resultat, missatge) VALUES (sysdate, procediment, parametresString, resultat, missatge);      
      COMMIT;
  END altaLogEntry;
  
  ---------------------------------------------------------------------------------------------------------   
  
  function consultaLogs(dataInici date, dataFinal date) return llistaLogsTable AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'consultaLogs'; 
    CURSOR llistaLogsCursor IS select * from Log where (data >= dataInici) and (data <= dataFinal);
    logActual logRec;
    llistaLogs llistaLogsTable := llistaLogsTable();
    i NUMBER;
    
    BEGIN
      
      open llistaLogsCursor;
      i := 1;
      LOOP
        FETCH llistaLogsCursor INTO logActual.id, logActual.data, logActual.procediment, logActual.parametres, logActual.resultat, logActual.missatge;
        EXIT WHEN llistaLogsCursor%NOTFOUND;
        
        llistaLogs.extend;
        llistaLogs(i) := logActual;
        i := i+1;
      END LOOP;
            
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      return llistaLogs;
      
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage); 
        
  end consultaLogs;
  
  --------------------------------------------------------------------------------------------------------- 
    
  procedure esborraLogs(dataInici date, dataFinal date) AS    
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'esborraLogs'; 
  
    BEGIN
    
      delete from Log where (data >= dataInici) and (data <= dataFinal);
      commit;
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
            
    EXCEPTION 
      WHEN others THEN      
      rollback;
      errorMessage := SQLERRM;
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage); 
  end esborraLogs;

END LOGPACKAGE;