create or replace PACKAGE NIVELLSPACKAGE AS 

  TYPE estadisticaRecord IS RECORD (
    id NUMBER,
    quantitat NUMBER
  );
  TYPE estadisticaTable is table of estadisticaRecord;
  
  procedure altaNivell(pNom VARCHAR2, pDificultat NUMBER, pNivellAnterior NUMBER);
  procedure modificaNivell(idNivell NUMBER, pDificultat NUMBER, pNivellAnterior NUMBER, pNom VARCHAR2); 
  procedure desbloquejaNivell(idNivell NUMBER, idPartida NUMBER);
  procedure altaAjuda(idNivell NUMBER, pNom VARCHAR2, pDescripcio VARCHAR2); 
  procedure comprarAjuda(idUsuari NUMBER, idAjuda NUMBER);
  function usuarisDesbloqueigNivells return estadisticaTable;
  function mitjanaVidesUsuariNivell return estadisticaTable;

END NIVELLSPACKAGE;

create or replace PACKAGE BODY NIVELLSPACKAGE AS

  procedure altaNivell(pNom VARCHAR2, pDificultat NUMBER, pNivellAnterior NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);

    BEGIN
  
      --Preparem el vector de parmetres per la taula Log
      parametres('nom') := pNom;
      parametres('dificultat') := pDificultat;
      parametres('nivellAnterior') := pNivellAnterior;
      
      --Executem la sentncia d'inserci de Nivell
      INSERT INTO Nivell(nom, dificultat, dataCreacio, nivellAnteriorId)
      VALUES (pNom, pDificultat, SYSDATE, pNivellAnterior);  
      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      logpackage.altaLogEntry(SYSDATE, 'altaNivell', parametres, 1, null);        
    
    EXCEPTION
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'altaNivell', parametres, 0, errorMessage); 

  END altaNivell;

  ---------------------------------------------------------------------------------------------------------
  
  procedure modificaNivell(idNivell NUMBER, pDificultat NUMBER, pNivellAnterior NUMBER, pNom VARCHAR2) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    noExisteixNivell EXCEPTION;
    existeixNivell NUMBER;
    noExisteixNivellException EXCEPTION;
    
    BEGIN
      parametres('idNivell') := idNivell;
      parametres('dificultat') := pDificultat;
      parametres('nivellAnterior') := pNivellAnterior;
      parametres('nom') := pNom;
      
      --Comprovem que existeixi el nivell
      SELECT count(*) into existeixNivell from Nivell where id=idNivell;
      
      if(existeixNivell=0) then
        raise noExisteixNivellException;
      end if;
      
      --Modifiquem els valors que no sn nuls            
      if(pNom is not null) then 
        UPDATE Nivell SET nom = pNom WHERE id = idNivell;
      end if;
      
      if(pDificultat is not null) then 
        UPDATE Nivell SET dificultat=pDificultat WHERE id=idNivell;
      end if;
      
      if(pNivellAnterior is not null) then 
        UPDATE Nivell SET nivellAnteriorId = pNivellAnterior WHERE id=idNivell;
      end if;      
      
      --S'ha de modificar tots els valors que no sn nuls o no modificarse'n cap.
      COMMIT;
      
      --Donem d'alta l'entrada al Log
      LogPackage.altaLogEntry(SYSDATE, 'modificaNivell', parametres, 1, null);
      
    EXCEPTION
      WHEN noExisteixNivellException THEN
        rollback;
        errorMessage := 'Error. El nivell amb id ' || idNivell || ' no existeix';        
        LogPackage.altaLogEntry(SYSDATE, 'modificaNivell', parametres, 0, errorMessage);     
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'modificaNivell', parametres, 0, errorMessage);
  END modificaNivell;

  ---------------------------------------------------------------------------------------------------------

  procedure desbloquejaNivell(idNivell NUMBER, idPartida NUMBER) AS
    -- Crearem una entrada a la taula NivellDesbloquejat amb les dades que ens arriben com a parmetre
    -- Amb la partida podem obtenir el idUsuari i tamb l'insertarem a la taula juntament amb l'idPartida
    -- per afavorir la velocitat i la claredat d'accs a les diferents taules relacionades.
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    diferentUsuariException EXCEPTION;
    idUsuari NUMBER;

    BEGIN
      -- Preparem el vector de parmetres per la taula Log
      parametres('idNivell') := idNivell;
      parametres('idPartida') := idPartida;
      
      -- Obtenim l'idUsuari a partir de la partida en la que es desbloqueja el nivell.
      SELECT usuariId into idUsuari from Partida where id = idPartida;
      
      --Executem la sentncia de creaci del registre de NivellDesbloquejat
      INSERT INTO NivellDesbloquejat(nivellId, usuariId, data, partidaId)
      VALUES (idNivell, idUsuari, SYSDATE, idPartida);  
      COMMIT;
      
      --Donem d'alta l'entrada Log amb tot correcte
      logpackage.altaLogEntry(SYSDATE, 'desbloquejaNivell', parametres, 1, null);        
    
    EXCEPTION
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'desbloquejaNivell', parametres, 0, errorMessage);
        
  END desbloquejaNivell;
  
  ---------------------------------------------------------------------------------------------------------
  
  procedure altaAjuda(idNivell NUMBER, pNom VARCHAR2, pDescripcio VARCHAR2) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'altaAjuda';
    BEGIN
      parametres('idNivell') := idNivell;
      parametres('nom') := pNom;
      parametres('descripcio') := pDescripcio;
      
      insert into Ajuda(nom, dataAlta, descripcio, nivellId) VALUES(pNom, SYSDATE, pDescripcio, idNivell);
      commit;
      logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      
    EXCEPTION
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);      
        
  END altaAjuda;
  
  ---------------------------------------------------------------------------------------------------------  
  
  procedure comprarAjuda(idUsuari NUMBER, idAjuda NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);  
    preuAjuda_ NUMBER;
    nomFuncio VARCHAR(100) := 'comprarAjuda';
    
  BEGIN
    parametres('idUsuari') := idUsuari;
    parametres('idAjuda') := idAjuda;
    
    -- Obtenim el preu de l'ajuda
    select preuAjuda into preuAjuda_ from Configuracio where id=1;
    
    -- Creem un nou registre a la taula AjudaUsuari.    
    insert into AjudaUsuari(ajudaId, usuariId, import, dataCompra) values (idAjuda, idUsuari, preuAjuda_, SYSDATE);
    commit;
    logpackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      
    EXCEPTION
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);          

  END comprarAjuda;
  
  --------------------------------------------------------------------------------------------------------- 
  
  function usuarisDesbloqueigNivells return estadisticaTable AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'usuarisDesbloqueigNivells';
    usuarisNivellsTable estadisticaTable := estadisticaTable();
    registre estadisticaRecord;
    CURSOR llistaNivellsCursor IS select id from Nivell;
    idNivellActual Nivell.id%type;
    i NUMBER; 
    
    BEGIN
      --Recorrem la llista de nivells i comptem el nmero d'usuaris que han desbloquejat aquell nivell
      open llistaNivellsCursor;
      i := 1;
      LOOP
        FETCH llistaNivellsCursor INTO idNivellActual;
        EXIT WHEN llistaNivellsCursor%NOTFOUND;
        usuarisNivellsTable.extend;
        usuarisNivellsTable(i).id := idNivellActual;
        select count(*) into usuarisNivellsTable(i).quantitat from NivellDesbloquejat where nivellId = idNivellActual;
        i := i+1;
      END LOOP;        
    
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      return usuarisNivellsTable;

    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);      
  
  end usuarisDesbloqueigNivells;
  
  --------------------------------------------------------------------------------------------------------- 
  
  function mitjanaVidesUsuariNivell return estadisticaTable AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'mitjanaVidesUsuariNivell';    
    CURSOR llistaNivellsCursor IS select id from Nivell;
    idNivellActual Nivell.id%type;
    videsNivellsTable estadisticaTable := estadisticaTable();
    i NUMBER; 
    numUsuaris NUMBER;
    videsGastades NUMBER;
    
    BEGIN 
    
    --Recorrem la llista de nivells per obtenir tots els usuaris que han desbloquejat aquell nivell
      open llistaNivellsCursor;
      i := 1;
      LOOP
        FETCH llistaNivellsCursor INTO idNivellActual;
        EXIT WHEN llistaNivellsCursor%NOTFOUND;
        --obtenim el total d'usuaris que han desbloquejat aquest nivell
        select count(*) into numUsuaris from NivellDesbloquejat where nivellId = idNivellActual;
        --obtenim el numero total de vides gastades en aquest nivell
        select count(*) into videsGastades from Vida where nivellGastadaId = idNivellActual;
        
        videsNivellsTable.extend;
        videsNivellsTable(i).id := idNivellActual;
        videsNivellsTable(i).quantitat := videsGastades / numUsuaris;
        i := i+1;
      END LOOP;
      
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      return videsNivellsTable;    
    
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage); 
        
  end mitjanaVidesUsuariNivell;   

END NIVELLSPACKAGE;