create or replace PACKAGE ParametritzacioPackage AS 

  PROCEDURE altaPais(nom IN VARCHAR2, abreviatura IN VARCHAR2);
  PROCEDURE modificarPreuVida(preu NUMBER);
  PROCEDURE modificarPreuAjuda(preu NUMBER);
  PROCEDURE definirVidesMnimes(numvides NUMBER);
  PROCEDURE altaPlataforma(plataforma VARCHAR2);
  
END ParametritzacioPackage;

create or replace PACKAGE BODY ParametritzacioPackage AS

  PROCEDURE altaPais(nom IN VARCHAR2, abreviatura IN VARCHAR2) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR(500);
    
    BEGIN
      --Preparem el vector de parmetres per la taula Log
      parametres('nom') := nom;
      parametres('abreviatura') := abreviatura;      

      --Executem la sentncia d'inserci de Pais
      INSERT INTO Pais(nom, abreviatura) VALUES (nom, abreviatura);
      COMMIT;
      --Donem d'alta l'entrada Log amb tot correcte
      logpackage.altaLogEntry(SYSDATE, 'altaPais', parametres, 1, null);        
    
    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'altaPais', parametres, 0, errorMessage);                            
  END altaPais;

------------------------------------------------------------------------------------------------

  PROCEDURE modificarPreuVida(preu NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR(500);  
  
    BEGIN
      parametres('preu') := preu;
      
      UPDATE Configuracio SET preuVida = preu WHERE id = 1;
      COMMIT;
      
      LogPackage.altaLogEntry(SYSDATE, 'modificarPreuVida', parametres, 1, NULL);

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'modificarPreuVida', parametres, 0, errorMessage);          
  END modificarPreuVida;
  
------------------------------------------------------------------------------------------------

  PROCEDURE modificarPreuAjuda(preu NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR(500);  
  
    BEGIN
      parametres('preu') := preu;
      
      UPDATE Configuracio SET preuAjuda = preu WHERE id = 1;
      COMMIT;
      
      LogPackage.altaLogEntry(SYSDATE, 'modificarPreuAjuda', parametres, 1, NULL);

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'modificarPreuAjuda', parametres, 0, errorMessage);    
  END modificarPreuAjuda;

------------------------------------------------------------------------------------------------

  PROCEDURE definirVidesMnimes(numVides NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR(500);  
    a NUMBER;
    valorNoEnter EXCEPTION;
  
    BEGIN
      parametres('numVides') := numVides;
      
      --Comprovem si el parametre d'entrada s un valor enter i si no ho s llencem una excepci
      if (MOD(numVides, trunc(numVides)) = 0) then      
        UPDATE Configuracio SET videsMinimes = numVides WHERE id = 1;
        COMMIT;
      else
        raise valorNoEnter;
      end if;    
      
      LogPackage.altaLogEntry(SYSDATE, 'definirVidesMinimes', parametres, 1, NULL);
      
    EXCEPTION
      WHEN valorNoEnter THEN
        ROLLBACK;
        errorMessage := 'El valor no s un nmero enter vlid';
        LogPackage.altaLogEntry(SYSDATE, 'definirVidesMinimes', parametres, 0, errorMessage);           
      WHEN others THEN
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'definirVidesMinimes', parametres, 0, errorMessage);   
  END definirVidesMnimes;

------------------------------------------------------------------------------------------------
    
  PROCEDURE altaPlataforma(plataforma VARCHAR2) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR(500);
    
    BEGIN
      --Preparem el vector de parmetres per la taula Log
      parametres('plataforma') := plataforma;

      --Executem la sentncia d'inserci de Pais
      INSERT INTO Plataforma(nom) VALUES (plataforma);
      COMMIT;
      --Donem d'alta l'entrada Log amb tot correcte
      logpackage.altaLogEntry(SYSDATE, 'altaPlataforma', parametres, 1, null);        
    
    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'altaPlataforma', parametres, 0, errorMessage);                            
  END altaPlataforma;

END ParametritzacioPackage;