create or replace PACKAGE USUARISPACKAGE AS 
  
  TYPE usuarisTable is table of Usuari%rowType;
  
  TYPE nivellDesbloquejat IS RECORD (
    idNivell NUMBER,
    dificultat NUMBER,
    nom VARCHAR(100),
    data DATE
  );
  
  TYPE nivellsTable is table of nivellDesbloquejat;
  
  TYPE estadisticaRecord IS RECORD (
    id NUMBER,
    quantitat NUMBER
  );
  TYPE estadisticaTable is table of estadisticaRecord;
  
  procedure altaUsuari(nom VARCHAR2, cognoms VARCHAR2, email VARCHAR2, username VARCHAR2, contrasenya VARCHAR2, pais NUMBER, sexe NUMBER, dataNaixement DATE);
  procedure modificaUsuari(idUsuari NUMBER, pNom VARCHAR2, pCognoms VARCHAR2, pEmail VARCHAR2, pContrasenya VARCHAR2, pPais NUMBER, pSexe NUMBER, pDataNaixement DATE);
  procedure baixaUsuari(idUsuari NUMBER);
  procedure enregistrarPartida(idUsuari NUMBER, idPlataforma NUMBER);
  procedure finalitzarPartida(idPartida NUMBER);
  function cercarUsuaris(pNom VARCHAR2, pCognoms VARCHAR2, pEmail VARCHAR2, pUsername VARCHAR2) return usuarisTable;
  function nivellsDesbloquejatsUsuari(idUsuari NUMBER) return nivellsTable;
  function videsComprades(idUsuari NUMBER) return NUMBER;
  function dinersGastatsEnVides(idUsuari NUMBER) return NUMBER;
  function ajudesComprades(idUsuari NUMBER) return NUMBER;
  function dinersGastatsEnAjudes(idUsuari NUMBER) return NUMBER;
  function usuarisPerSexe return estadisticaTable;
  function usuarisPerPais return estadisticaTable;
  function partidesPerPlataforma return estadisticaTable;

END USUARISPACKAGE;

create or replace PACKAGE BODY USUARISPACKAGE AS

  procedure altaUsuari(nom VARCHAR2, cognoms VARCHAR2, email VARCHAR2, username VARCHAR2, contrasenya VARCHAR2, pais NUMBER, sexe NUMBER, dataNaixement DATE) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    patroEmail VARCHAR2(100);
    formatEmailIncorrecte EXCEPTION;
    idSexeIncorrecte EXCEPTION;
   
    BEGIN
      --Preparem el vector de parmetres per la taula Log
      parametres('nom') := nom;
      parametres('cognoms') := cognoms;
      parametres('email') := email;
      parametres('userame') := username;
      parametres('contrasenya') := contrasenya;
      parametres('pais') := pais;
      parametres('sexe') := sexe;
      parametres('dataNaixement') := dataNaixement;

      --Validem si el format de correu electrnic s correcte
      patroEmail := '^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$';
      if(not regexp_like(email, patroEmail)) then
        raise formatEmailIncorrecte;        
      end if;
      
      --Validem si el sexe s 0 o 1
      if((sexe != 0) and (sexe != 1)) then
        raise idSexeIncorrecte;
      end if;
      
      --Executem la sentncia d'inserci d'Usuari
      INSERT INTO Usuari(nom, cognoms, email, contrasenya, dataNaixement, sexe, dataAlta, actiu, paisId, username)
      VALUES (nom, cognoms, email, contrasenya, dataNaixement, sexe, SYSDATE, 1, pais, username);  
      COMMIT;
      --Donem d'alta l'entrada Log amb tot correcte
      logpackage.altaLogEntry(SYSDATE, 'altaUsuari', parametres, 1, null);        
    
    EXCEPTION
      WHEN formatEmailIncorrecte THEN
        errorMessage := 'Error. El format de correu electrnic s incorrecte';
        LogPackage.altaLogEntry(SYSDATE, 'altaUsuari', parametres, 0, errorMessage); 
      WHEN idSexeIncorrecte THEN
        errorMessage := 'Error. El sexe ha de ser 0 o 1.';
        LogPackage.altaLogEntry(SYSDATE, 'altaUsuari', parametres, 0, errorMessage);         
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'altaUsuari', parametres, 0, errorMessage); 
  END altaUsuari;

---------------------------------------------------------------------------------------------------------

--  ModificaUsuari modificar els camps que no siguin null amb el valor d'entrada del parmetre

  procedure modificaUsuari(idUsuari NUMBER, pNom VARCHAR2, pCognoms VARCHAR2, pEmail VARCHAR2, pContrasenya VARCHAR2, pPais NUMBER, pSexe NUMBER, pDataNaixement DATE) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    patroEmail VARCHAR2(100);
    formatEmailIncorrecte EXCEPTION;
    idSexeIncorrecte EXCEPTION;
    noExisteixUsuari EXCEPTION;
    existeixUsuari NUMBER;
    
    BEGIN
      parametres('idUsuari') := idUsuari;
      parametres('nom') := pNom;
      parametres('cognoms') := pCognoms;
      parametres('email') := pEmail;
      parametres('contrasenya') := pContrasenya;
      parametres('pais') := pPais;
      parametres('sexe') := pSexe;
      parametres('dataNaixement') := pDataNaixement;
      
      --Comprovem que existeix l'usuari
      SELECT count(*) into existeixUsuari from Usuari where id=idUsuari;
      if(existeixUsuari=0) then
        raise noExisteixUsuari;
      end if;
      
      --Modifiquem els valors que no sn nuls            
      if(pNom is not null) then 
        UPDATE Usuari SET nom = pNom WHERE id = idUsuari;
      end if;
      
      if(pCognoms is not null) then 
        UPDATE Usuari SET cognoms=pCognoms WHERE id=idUsuari;
      end if;
      
      if(pEmail is not null) then 
        --Validem si el format de correu electrnic s correcte
        patroEmail := '^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$';
        if(not regexp_like(pEmail, patroEmail)) then
          raise formatEmailIncorrecte;        
        end if;
        UPDATE Usuari SET email = pEmail WHERE id=idUsuari;
      end if;
      
      if(pContrasenya is not null) then 
        UPDATE Usuari SET contrasenya = pContrasenya WHERE id=idUsuari;
      end if;      
      
      if(pPais is not null) then 
        UPDATE Usuari SET paisId=pPais WHERE id=idUsuari;
      end if;
      
      if(pSexe is not null) then 
        --Validem si el sexe s 0 o 1
        if((pSexe != 0) and (pSexe != 1)) then
          raise idSexeIncorrecte;
        end if;      
        UPDATE Usuari SET sexe=pSexe WHERE id=idUsuari;
      end if;
      
      if(pDataNaixement is not null) then 
        UPDATE Usuari SET dataNaixement=pDataNaixement WHERE id=idUsuari;
      end if;
      
      --S'ha de modificar tots els valors que no sn nuls o no modificarse'n cap.
      COMMIT;
      
      --Donem d'alta l'entrada al Log
      LogPackage.altaLogEntry(SYSDATE, 'modificaUsuari', parametres, 1, null);
      
    EXCEPTION
      WHEN noExisteixUsuari THEN
        rollback;
        errorMessage := 'Error. L''usuari amb id ' || idUsuari || ' no existeix';        
        LogPackage.altaLogEntry(SYSDATE, 'modificaUsuari', parametres, 0, errorMessage);     
      WHEN formatEmailIncorrecte THEN
        rollback;
        errorMessage := 'Error. El format de correu electrnic s incorrecte';        
        LogPackage.altaLogEntry(SYSDATE, 'modificaUsuari', parametres, 0, errorMessage); 
      WHEN idSexeIncorrecte THEN
        rollback;
        errorMessage := 'Error. El sexe ha de ser 0 o 1.';        
        LogPackage.altaLogEntry(SYSDATE, 'modificaUsuari', parametres, 0, errorMessage);         
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'modificaUsuari', parametres, 0, errorMessage);
            
  END modificaUsuari;

---------------------------------------------------------------------------------------------------------

  procedure baixaUsuari(idUsuari NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    noExisteixUsuari EXCEPTION;
    existeixUsuari NUMBER;
  
    BEGIN
      parametres('idUsuari') := idUsuari;
      
      --Comprovem que existeix l'usuari
      SELECT count(*) into existeixUsuari from Usuari where id=idUsuari;
      if(existeixUsuari=0) then
        raise noExisteixUsuari;
      end if;      
      
      UPDATE Usuari SET actiu = 0 WHERE id = idUsuari;
      COMMIT;
      
      LogPackage.altaLogEntry(SYSDATE, 'baixaUsuari', parametres, 1, null);
    EXCEPTION
      WHEN noExisteixUsuari THEN
        rollback;
        errorMessage := 'Error. L''usuari amb id ' || idUsuari || ' no existeix';        
        LogPackage.altaLogEntry(SYSDATE, 'baixaUsuari', parametres, 0, errorMessage);             
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'baixaUsuari', parametres, 0, errorMessage);      
  END baixaUsuari;
  
  ---------------------------------------------------------------------------------------------------------
  
  procedure enregistrarPartida(idUsuari NUMBER, idPlataforma NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    BEGIN
      parametres('idUsuari') := idUsuari;
      parametres('idPlataforma') := idPlataforma;
      
      INSERT INTO Partida(dataInici, dataFinal, usuariId, plataformaId) VALUES (SYSDATE, null, idUsuari, idPlataforma);
      commit;
      
      LogPackage.altaLogEntry(SYSDATE, 'enregistrarPartida', parametres, 1, 0);
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'enregistrarPartida', parametres, 0, errorMessage);            
      
  END enregistrarPartida;
  
 ---------------------------------------------------------------------------------------------------------

  procedure finalitzarPartida(idPartida NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    noExisteixPartida EXCEPTION;
    partidaJaFinalitzadaException EXCEPTION;
    existeixPartida NUMBER;
    partidaRec Partida%rowtype;
    
    BEGIN
      parametres('idPartida') := idPartida;
      
      --Comprovem que existeix la partida
      --Si no existeix la partida el segent select llenar una excepcio NOTDATAFOUND
      SELECT * into partidaRec FROM Partida WHERE id=idPartida;            
                 
      --Comprovem que la partida no estigui finalitzada.
      if(partidaRec.dataFinal is not null) then
        raise partidaJaFinalitzadaException;
      end if;
      
      UPDATE Partida SET dataFinal = SYSDATE WHERE id = idPartida;
      
      LogPackage.altaLogEntry(SYSDATE, 'finalitzaPartida', parametres, 1, null);
      
    EXCEPTION
      WHEN noExisteixPartida THEN
        rollback;
        errorMessage := 'Error. La partida amb id ' || idPartida || ' no existeix';        
        LogPackage.altaLogEntry(SYSDATE, 'finalitzaPartida', parametres, 0, errorMessage);
      WHEN partidaJaFinalitzadaException THEN
        rollback;
        errorMessage := 'Error. La partida amb id ' || idPartida || ' ja estava finalitzada';        
        LogPackage.altaLogEntry(SYSDATE, 'finalitzaPartida', parametres, 0, errorMessage);        
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'finalitzaPartida', parametres, 0, errorMessage);   
  END finalitzarPartida;
  
---------------------------------------------------------------------------------------------------------  

  function cercarUsuaris(pNom VARCHAR2, pCognoms VARCHAR2, pEmail VARCHAR2, pUsername VARCHAR2) return usuarisTable AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    sqlStr VARCHAR2(1000);
    TYPE cursorType is REF CURSOR;
    resultatCur cursorType;
    resultatTab usuarisTable := usuarisTable();
    usuariRec Usuari%rowtype;
    i NUMBER;
    
    BEGIN
      parametres('nom') := pNom;
      parametres('cognoms') := pCognoms;
      parametres('email') := pEmail;
      parametres('username') := pUsername;
      

      --Construm la sentncia sql en funci dels parmetres de la funci.
      sqlStr := 'select * from Usuari where ';
      
      if(pNom is not null) then
        sqlStr := sqlStr || '(nom like ''%' || pNom || '%'')'; 
      end if;
      
      if(pCognoms is not null) then
        sqlStr := sqlStr || ' and (cognoms like ''%' || pCognoms || '%'')'; 
      end if;
      
      if(pEmail is not null) then
        sqlStr := sqlStr || ' and (email like ''%' || pEmail || '%'')'; 
      end if;
      
      if(pUsername is not null) then
        sqlStr := sqlStr || ' and (username like ''%' || pUsername || '%'')'; 
      end if;

      
      open resultatCur for sqlStr;
      
      --Recorrerem el cursor i colocarem els valors retornats a la taula que retornar la funci
      i := 1;
      LOOP
        FETCH resultatCur INTO usuariRec;
        EXIT WHEN resultatCur%NOTFOUND;
        resultatTab.extend;
        resultatTab(i) := usuariRec;
        i := i+1;
      END LOOP;      
            
      close resultatCur;
      
      LogPackage.altaLogEntry(SYSDATE, 'cercarUsuaris', parametres, 1, null);     
      RETURN resultatTab;
      
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'cercarUsuaris', parametres, 0, errorMessage);     
  END cercarUsuaris;

  ---------------------------------------------------------------------------------------------------------  

  function nivellsDesbloquejatsUsuari(idUsuari NUMBER) return nivellsTable AS

    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    resultatTab nivellsTable := nivellsTable();
    
    CURSOR nivellsDesbloquejatsCur IS
        select Nivell.id, Nivell.dificultat, Nivell.nom, NivellDesbloquejat.data 
        from NivellDesbloquejat, Nivell 
        where NivellDesbloquejat.nivellId=Nivell.id and NivellDesbloquejat.usuariId = idUsuari;
    nivellRec nivellsDesbloquejatsCur%rowtype;
    i NUMBER;
        
    BEGIN
      parametres('idUsuari') := idUsuari;
      
      open nivellsDesbloquejatsCur;
      
      --Recorrerem el cursor i colocarem els valors retornats a la taula que retornar la funci
      i := 1;
      LOOP
        FETCH nivellsDesbloquejatsCur INTO nivellRec;
        EXIT WHEN nivellsDesbloquejatsCur%NOTFOUND;
        resultatTab.extend;
        resultatTab(i) := nivellRec;        
        i := i+1;
      END LOOP;      
            
      close nivellsDesbloquejatsCur;
      
      LogPackage.altaLogEntry(SYSDATE, 'nivellsDesbloquejatsUsuari', parametres, 1, null);     
      RETURN resultatTab;
      
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'nivellsDesbloquejatsUsuari', parametres, 0, errorMessage);
  END nivellsDesbloquejatsUsuari;  
    
  --------------------------------------------------------------------------------------------------------- 
  
  function videsComprades(idUsuari NUMBER) return NUMBER AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'videsComprades';
    numVidesComprades NUMBER;
    
    BEGIN
      parametres('idUsuari') := idUsuari;
      
      select count(*) into numVidesComprades from Vida 
      where (usuariPropietariId = idUsuari) and tipusVidaId = 3; 
      
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      
      return numVidesComprades;
    
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);    
  END;
  
  --------------------------------------------------------------------------------------------------------- 
  
  function dinersGastatsEnVides(idUsuari NUMBER) return NUMBER AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dinersGastatsEnVides';
    dinersGastats NUMBER;
    
    BEGIN
      parametres('idUsuari') := idUsuari;
      
      select sum(import) into dinersGastats from Vida 
      where (usuariPropietariId = idUsuari) and tipusVidaId = 3; 
      
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      
      return dinersGastats;
    
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);    
  END;
  
  --------------------------------------------------------------------------------------------------------- 
  
  function usuarisPerSexe return estadisticaTable AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'usuarisPerSexe';
    usuarisPerSexeTable estadisticaTable := estadisticaTable();
    registre estadisticaRecord;
        
    BEGIN
      
      --Comptem la quantitat de sexes = 0 i l'assignem a la taula de retorn
      registre.id := 0;
      select count(*) into registre.quantitat from Usuari where sexe = 0;      
      usuarisPerSexeTable.extend;
      usuarisPerSexeTable(1) := registre;
      
      --Comptem la quantitat de sexes = 1 i l'assignem a la taula de retorn
      registre.id := 1;
      select count(*) into registre.quantitat from Usuari where sexe = 1;      
      usuarisPerSexeTable.extend;
      usuarisPerSexeTable(2) := registre;
    
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      return usuarisPerSexeTable;
    
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);
  END;
  
  --------------------------------------------------------------------------------------------------------- 
  
  function usuarisPerPais return estadisticaTable AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'usuarisPerPais';
    usuarisPerPaisTable estadisticaTable := estadisticaTable();
    registre estadisticaRecord;
    CURSOR llistaPaisosCursor IS
      select id from Pais;
    idPaisActual Pais.id%type;
    i NUMBER; 
        
    BEGIN      
      --Recorrem la llista de paisos i comptem el nmero d'usuaris d'aquell pais
      open llistaPaisosCursor;
      i := 1;
      LOOP
        FETCH llistaPaisosCursor INTO idPaisActual;
        EXIT WHEN llistaPaisosCursor%NOTFOUND;
        usuarisPerPaisTable.extend;
        usuarisPerPaisTable(i).id := idPaisActual;
        select count(*) into usuarisPerPaisTable(i).quantitat from Usuari where PaisId = idPaisActual;
        i := i+1;
      END LOOP;        
    
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      return usuarisPerPaisTable;
    
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);
  END;
  
  --------------------------------------------------------------------------------------------------------- 
  
  function partidesPerPlataforma return estadisticaTable AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'partidesPerPlataforma';
    partidesPerPlataformaTable estadisticaTable := estadisticaTable();
    registre estadisticaRecord;
    CURSOR llistaPlataformesCursor IS
      select id from Plataforma;
    idPlataformaActual Plataforma.id%type;
    i NUMBER; 
        
    BEGIN      
      --Recorrem la llista de paisos i comptem el nmero d'usuaris d'aquell pais
      open llistaPlataformesCursor;
      i := 1;
      LOOP
        FETCH llistaPlataformesCursor INTO idPlataformaActual;
        EXIT WHEN llistaPlataformesCursor%NOTFOUND;
        partidesPerPlataformaTable.extend;
        partidesPerPlataformaTable(i).id := idPlataformaActual;
        select count(*) into partidesPerPlataformaTable(i).quantitat from Partida where PlataformaId = idPlataformaActual;
        i := i+1;
      END LOOP;        
    
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      return partidesPerPlataformaTable;
    
    EXCEPTION
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);
  END;

  ---------------------------------------------------------------------------------------------------------
  
  function ajudesComprades(idUsuari NUMBER) return NUMBER AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'ajudesComprades';
    numAjudesComprades NUMBER;
    existeixUsuari NUMBER;
    usuariNoExisteixException EXCEPTION;
    
    BEGIN
      parametres('idUsuari') := idUsuari;
      
      --Comprovem que l'usuari existeixi.
      select count(*) into existeixUsuari from Usuari where id=idUsuari;
      
      if(existeixUsuari = 0) then
        raise usuariNoExisteixException;
      end if;
      
      select count(*) into numAjudesComprades from AjudaUsuari 
      where usuariId = idUsuari;
      
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      
      return numAjudesComprades;
    
    EXCEPTION
      WHEN usuariNoExisteixException THEN
        rollback;
        errorMessage := 'No existeix l''usuari amb id ' || idUsuari;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);            
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);    
  END;
  
  ---------------------------------------------------------------------------------------------------------  
  
  function dinersGastatsEnAjudes(idUsuari NUMBER) return NUMBER AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    nomFuncio VARCHAR(100) := 'dinersGastatsEnAjudes';
    dinersGastats NUMBER;
    existeixUsuari NUMBER;
    usuariNoExisteixException EXCEPTION;
    
    BEGIN
      parametres('idUsuari') := idUsuari;
      
      --Comprovem que l'usuari existeixi.
      select count(*) into existeixUsuari from Usuari where id=idUsuari;
      
      if(existeixUsuari = 0) then
        raise usuariNoExisteixException;
      end if;
      
      select sum(import) into dinersGastats from AjudaUsuari 
      where usuariId = idUsuari;
      
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null);
      
      return dinersGastats;
    
    EXCEPTION
      WHEN usuariNoExisteixException THEN
        rollback;
        errorMessage := 'No existeix l''usuari amb id ' || idUsuari;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);            
      WHEN others THEN      
        rollback;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);   
      
  END;

END USUARISPACKAGE;