create or replace PACKAGE VIDESPACKAGE AS 

  procedure carregarVides(idUsuari NUMBER);
  procedure comprarVida(idUsuari NUMBER);
  procedure cedirVida(idUsuariDonant NUMBER, idUsuariReceptor NUMBER);
  procedure gastarVida(idUsuari NUMBER, idNivell NUMBER);
  function consultarVidesIntercanviades(dataInici DATE, dataFinal DATE) return NUMBER;
END VIDESPACKAGE;

create or replace PACKAGE BODY VIDESPACKAGE AS

  -- carregarVides: Carregar tantes vides de tipus lliure a l'usuari com marca l'atribut videsMinimes.
  procedure carregarVides(idUsuari NUMBER) AS    
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);
    videsRestants NUMBER;
    videsMinimes_ NUMBER;
    i NUMBER;

    BEGIN    
      --Preparem el vector de parmetres per la taula Log
      parametres('idUsuari') := idUsuari;
      
      -- Obtenir les vides mnimes que li queden a l'usuari.
      select count(*) into videsRestants from Vida 
      where (usuariPropietariId = idUsuari) and (tipusVidaId = 1) and (nivellGastadaId is null);
      
      -- Obtenim les vides mnimes
      select videsMinimes into videsMinimes_ from Configuracio where id=1;
               
      -- Crear tantes vides de tipus lliure com li manquin a l'usuari i relacionar-les amb ell.
      if(videsRestants < videsMinimes_) then
        i := videsRestants;
        while(i < videsMinimes_) loop
          insert into Vida(dataCarrega, tipusVidaId, import, usuariPropietariId) values (SYSDATE, 1, 0, idUsuari);  
          i := i +1;
        end loop;
      end if;
      
      commit;
      --Donem d'alta l'entrada Log amb tot correcte
      logpackage.altaLogEntry(SYSDATE, 'carregarVides', parametres, 1, null);        
    
    EXCEPTION
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'carregarVides', parametres, 0, errorMessage); 
            
  END carregarVides;
  
  ---------------------------------------------------------------------------------------------------
  -- ComprarVida dna d'alta una vida de tipus comprada amb el seu corresponent import en el moment
  -- de la compra i l'assigna a l'usuari que la compra  

  procedure comprarVida(idUsuari NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);  
    preuVida_ NUMBER;
  
    BEGIN
       --Preparem el vector de parmetres per la taula Log
      parametres('idUsuari') := idUsuari;
      
      -- Obtenim el preu de la vida
      select preuVida into preuVida_ from Configuracio where id=1;
      
      -- Donem d'alta la vida comprada
      insert into Vida(dataCarrega, tipusVidaId, import, usuariPropietariId) values (SYSDATE, 3, preuVida_, idUsuari);  
      
      commit;
      
      --Donem d'alta l'entrada Log amb tot correcte
      logpackage.altaLogEntry(SYSDATE, 'comprarVida', parametres, 1, null);        
    
    EXCEPTION
      WHEN others THEN      
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, 'comprarVida', parametres, 0, errorMessage);          

  END comprarVida;

  ---------------------------------------------------------------------------------------------------
  -- cedirVida dona d'alta una vida de tipus cedida per l'usuari receptor i es posa com a gastada una de les vides
  -- de l'usuari donant
  
  procedure cedirVida(idUsuariDonant NUMBER, idUsuariReceptor NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);  
    numVidesUsuariDonant NUMBER;
    noVidesException Exception;
    nomFuncio VARCHAR(100) := 'cedirVida';
    idVidaBaixa Vida.id%type;
    CURSOR videsUsuari IS
      select id from Vida where (usuariPropietariId = idUsuariDonant) and (dataGastada is null);
    
    BEGIN
      -- Preparem el vector de parmetres per la taula Log
      parametres('idUsuariDonant') := idUsuariDonant;
      parametres('idUsuariReceptor') := idUsuariReceptor;
      
      -- Obtenim els ids de les vides de l'usuari
      open videsUsuari;    
      fetch videsUsuari into idVidaBaixa;

      -- Comprovem que l'usuari donant disposi de vides per donar
      if(idVidaBaixa is null) then
        raise noVidesException;
      end if;
      
      -- Donem d'alta la vida cedida
      insert into Vida(dataCarrega, tipusVidaId, import, usuariPropietariId, usuariCessioId) values (SYSDATE, 2, 0, idUsuariReceptor, idUsuariDonant);
      
      -- Posem com a gastada una de las vides de l'usuari donant
      update Vida set dataGastada = SYSDATE where id = idVidaBaixa;
      
      commit;
      
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null); 

      EXCEPTION
        WHEN noVidesException THEN
          ROLLBACK;
          errorMessage := 'L''usuari donant no t cap vida';
          LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);             
        WHEN others THEN
          ROLLBACK;
          errorMessage := SQLERRM;
          LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);   
        
  END cedirVida;
  
  ---------------------------------------------------------------------------------------------------
  
  -- GastarVida posar data de gastada i nivell a una de les vides de l'usuari. Primer es gastaran les vides
  -- Lliures, desprs les cedides i per ltim les comprades.
  procedure gastarVida(idUsuari NUMBER, idNivell NUMBER) AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);  
    nomFuncio VARCHAR(100) := 'gastarVida';
    idVidaBaixa Vida.id%type;
    noVidesException Exception;
    usuariNoExisteixException Exception;
    nivellNoExisteixException Exception;
    c NUMBER;
    
    CURSOR videsLliures IS  -- Per comprovar si l'usuari t vides lliures
      select id from Vida 
      where (usuariPropietariId = idUsuari) and (dataGastada is null) and (tipusVidaId = 1);
    CURSOR videsCedides IS  -- Per comprovar si l'usuari t vides cedides
      select id from Vida 
      where (usuariPropietariId = idUsuari) and (dataGastada is null) and (tipusVidaId = 2);
    CURSOR videsComprades IS  -- Per comprovar si l'usuari t vides comprades
      select id from Vida 
      where (usuariPropietariId = idUsuari) and (dataGastada is null) and (tipusVidaId = 3);      
    
    BEGIN
      -- Preparem el vector de parmetres per la taula Log
      parametres('idUsuari') := idUsuari;
      parametres('idNivell') := idNivell;
      
      -- Comprovem si existeixen l'usuari i el nivell
      select count(*) into c from Usuari where id=idUsuari;
      if(c=0) then
        raise usuariNoExisteixException;
      end if;

      select count(*) into c from Nivell where id=idNivell;
      if(c=0) then
        raise NivellNoExisteixException;
      end if;      
      
      open videsLliures;    
      fetch videsLliures into idVidaBaixa;
      
      if(idVidaBaixa is null) then
        -- L'usuari no t cap vida lliure. Comprovem si en t de cedides
        open videsCedides;
        fetch videsCedides into idVidaBaixa;
        if(idVidaBaixa is null) then
          -- L'usuari no t cap vida cedida. Comprovem si en t de comprades
          open videsComprades;
          fetch videsComprades into idVidaBaixa;
          if(idVidaBaixa is null) then
            -- L'usuari no t cap vida comprada. Llencem una excepci.
            raise noVidesException;
          end if;
        end if;
      end if;

      -- Si s'ha arribat fins aqu vol dir que idVidaBaixa cont l'id de la vida a gastar.
      -- Posem com a gastada la vida amb id = idVidaBaixa
      update Vida set dataGastada = SYSDATE, nivellGastadaId = idNivell where id = idVidaBaixa;
      
      commit;
      
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null); 

      EXCEPTION
        WHEN usuariNoExisteixException THEN
          ROLLBACK;
          errorMessage := 'L''usuari amb id ' || idUsuari || ' no existeix';
          LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);             
        WHEN nivellNoExisteixException THEN
          ROLLBACK;
          errorMessage := 'El nivell amb id ' || idNivell || ' no existeix';
          LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);             
        WHEN noVidesException THEN
          ROLLBACK;
          errorMessage := 'L''usuari amb id ' || idUsuari || ' no t cap vida';
          LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage);             
        WHEN others THEN
          ROLLBACK;
          errorMessage := SQLERRM;
          LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage); 
  END gastarVida;
  
  function consultarVidesIntercanviades(dataInici DATE, dataFinal DATE) return NUMBER AS
    parametres LogPackage.parametresList;
    errorMessage VARCHAR2(500);  
    nomFuncio VARCHAR(100) := 'consultarVidesIntercanviades';
    videsIntercanviades NUMBER;
    
    BEGIN
      
      select count(*) into videsIntercanviades from Vida 
      where (tipusVidaId = 2) and (dataCarrega >= dataInici) and (dataCarrega <=dataFinal); 
        
      commit;
      LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 1, null); 
      return videsIntercanviades;
      
    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        errorMessage := SQLERRM;
        LogPackage.altaLogEntry(SYSDATE, nomFuncio, parametres, 0, errorMessage); 
    
  END consultarVidesIntercanviades;
  
END VIDESPACKAGE;